{ 4I20 low - low level 4I20 I/O routines }

{ 4I20 LatchBits }
const

AddMask = $1F;
WEnaBit    = $20;
PowerBit   = $40;
DataBit = $80;

DefaultPort = $300;

var
I20Port : word;

procedure DisableInterrupts;
begin
  inline($FA);                                 { CLI }
end;

procedure EnableInterrupts;
begin
  inline($FB);                                 { STI }
end;

procedure WaitABit;
var
loops : longint;
dummy : byte;
begin
  for loops := 1 to 5000 do dummy := port[I20Port];
  { this is to wait for power to come up on isolated side }
end;


procedure Init4I20;
begin
  { This procedure does a first time init of a 4I20 }
  port[I20Port] := 0;
  WaitABit;  { wait for power to go down; }
  port[I20Port] := PowerBit;
  WaitABit;  { for power to come up }
  { We need all isolated bits to toggle once before use }
  port[I20Port] := PowerBit or AddMask or WEnaBit or DataBit;
  { This should write to latch 31 (non-existant) }
  port[I20Port] := PowerBit; { this should remove reset }
  port[I20Port] := PowerBit or AddMask or DataBit;  { this is a no-op read }
  { This last output was added to solve an intermittant problem with startup }
  { when the 4I20 is powered down but the host stays up 1/14/95 PCW }
end;

procedure PowerDown4I20;
begin
  { This procedure shuts down remote power and resets a 4I20 }
  port[I20Port] := 0;
end;

procedure BitOut(TheBit : byte; biton : boolean);
var
data : byte;
begin
  data := TheBit and AddMask;
  data := data or PowerBit or WEnaBit;
  if biton then data := data or DataBit;
  port[I20Port] := data;
end;

function BitIn(TheBit : byte) : boolean;
var
data : byte;
begin
  data := TheBit and AddMask;
  data := data or PowerBit;
  port[I20Port] := data;
  if (port[I20Port] and DataBit) <> 0 then BitIn := true else BitIn := false;
end;

function CardThere : boolean;
var thebit : byte;
begin
  Init4I20;
  CardThere := true;
  { This test works by reading unused bits (>23 & <32) in the 4I20   }
  { These bits follow the LSB of the bit address (odd ones are high) }
  { This test does not touch any output bits }
  for thebit := 24 to 31 do
  begin
    if BitIn(thebit) <> ((thebit and 1) = 1) then CardThere := false;
  end;
end;

function CardWarble : boolean;
var thebit : byte;
begin
  CardWarble := true;
  { This test works by reading unused bits (>23 & <32) in the 4I20   }
  { These bits follow the LSB of the bit address (odd ones are high) }
  { This test does not touch any output bits }
  DisableInterrupts;
  for thebit := 24 to 31 do
  begin
    if BitIn(thebit) <> ((thebit and 1) = 1) then CardWarble := false;
  end;
  EnableInterrupts;
end;
